
! Copyright (C) 2012 S. Dugdale and D. Ernsting.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.

subroutine compton
use modmain
implicit none





return
end subroutine

